// The MIT License (MIT) - Copyright (c) 2016 Carlos Vidal
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
@testable import EasyPeasy

class UIView_EasyTests: XCTestCase {
    
    override func setUp() {
        super.setUp()
    }
    
    override func tearDown() {
        super.tearDown()
    }
    
    func testThatReloadMethodReinstallsTheAttributesApplied() {
        // given
        let superview = UIView(frame: CGRectMake(0, 0, 400, 1000))
        let viewA = UIView(frame: CGRectZero)
        superview.addSubview(viewA)
        let viewB = UIView(frame: CGRectZero)
        superview.addSubview(viewB)
        var shouldApply = false
        viewA <- Width(120).when { shouldApply }
        viewA <- Height(120)
        XCTAssertTrue((superview.constraints.filter{$0.easy_attribute != nil}).count == 1)
        
        // when
        shouldApply = true
        viewA.easy_reload()
        
        // then
        XCTAssertTrue((superview.constraints.filter{$0.easy_attribute != nil}).count == 2)
    }
    
    func testThatCompoundAttributesAreNotReturnedAndOnlyRegularAttributesStoredInView() {
        // given
        let superview = UIView(frame: CGRectMake(0, 0, 400, 1000))
        let viewA = UIView(frame: CGRectZero)
        superview.addSubview(viewA)

        // when
        let attributes = viewA <- Edges(10)
        
        // then
        XCTAssertTrue(attributes.count == 4)
        for attribute in attributes {
            XCTAssertFalse(attribute is CompoundAttribute)
        }
        XCTAssertTrue(superview.easy_attributes.count == 4)
        for attribute in superview.easy_attributes {
            XCTAssertFalse(attribute is CompoundAttribute)
        }
    }
    
}
